export class AbstractElement extends HTMLElement {
    constructor(renderFunc, shadow = false, mode = 'open') {
        super();
        this.connected = false;
        this.attr = {};
        this.attach = (container) => {
            if (this.connected) {
                renderFunc(container, this.render());
            }
        };
        if (shadow) {
            this.attach = this.attach.bind(null, this.attachShadow({ mode }));
        }
        else {
            this.attach = this.attach.bind(null, this);
        }
    }
    set state(newState) {
        this._scope = newState || this.state;
        this.attach();
    }
    get state() {
        return this._scope;
    }
    static get observedAttributes() {
        return Object.keys(this.attributes).map(key => this.attributes[key]);
    }
    connectedCallback() {
        this.connected = true;
        this.attach();
    }
    attributeChangedCallback(name, oldValue, newValue) {
        if (oldValue !== newValue && this.attr[name] !== newValue) {
            this.attr[name] = newValue;
            this.attach();
        }
    }
}
AbstractElement.attributes = {};
//# sourceMappingURL=abstract-element.js.map